/*
 * Copyright (c) 2010 Matthias Bach,
 *                    bach@compeng.uni-frankfurt.de
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * Check whether iteration from the give start value will converge to 0.
 */
uchar mandelbrot( const float xi, const float yi, const float max_abs, const uchar max_iter )
{
	uchar iter = 0;
	float x,y,yy,xx;  
	
	x = xi;
	y = yi;
	
	yy = y * y;
	xx = x * x;
	
	while( ((xx + yy) < max_abs * max_abs) && (iter < max_iter) ) {
	      y = 2.0f*x*y + yi;
	      x = xx - yy + xi;
	      ++iter;
	      yy = y * y;
	      xx = x * x; 
	}
	return iter;
}

__kernel void mandelbrot_gpu(__global __write_only uchar3 *dest, uint rows, uint cols)
{
	int i = get_global_id(0);
	int j = get_global_id(1);
	int col;
	
	const uchar max_iter = 255;
	float max_abs = 2.0f;
	float x0 =-0.5f;
	float y0 = 0.0f; 
	float sx = 2.0f / cols;
	float sy = 2.0f / rows;
	float x,y;

	if( i >= cols || j >= rows ) return;

	//TODO
	//Call mandelbrot function with appropriat parameters
}
